#!/usr/bin/env perl

use strict;
use warnings;
use feature ":5.10";

use Moose::Util qw/find_meta does_role search_class_by_role/;
use Module::Find;

my @event_classes = useall state51::Logger::Event;

my $dot;
foreach my $class (sort @event_classes) {
	$dot .= class_as_dot($class);
}

$dot = "digraph test {
	graph [ratio=fill];
	node [shape=record];
".$dot."\n}";

say $dot;
# dot -Tpng event.dot > event.png

sub class_as_dot {
	my ($class) = @_;

	#warn is_role($class);

	my $meta = find_meta($class);
	# you can't make roles directly, so they're not useful to diagram?
	return '' if is_role($class);

	my $node_name = $class;
	$node_name =~ s!::!_!g;

	my $class_label = $class;
	$class_label =~ s!state51::Logger::!!;

	my $attributes;

	if (is_role($class)) { # you can't get_attribute_list on non-roles
		$class_label .= ' (role)';
		# returns a list of names
		$attributes = [$meta->get_attribute_list];
	}
	else {
		$attributes = [map {$_->name()} $meta->get_all_attributes];
	}

	$attributes = join("|", sort @{$attributes});
	$attributes = '<B>'.$class_label.'</B>|' . $attributes;

	my $result = '        '.$node_name.' [shape=record, label=<{'.$attributes.'}>]'.";\n";

	return $result;
}

sub is_role {
    my $package_or_obj = shift;

    my $meta = find_meta($package_or_obj);
    return if not $meta;
    return $meta->isa('Moose::Meta::Role');
}
