package state51::Logger::Event::ImageResize;
use Moose;
use MooseX::Storage ();
use MooseX::Types::Moose qw/ Str Int /;
use namespace::autoclean;

extends 'state51::Logger::Event';

# image we start with
has from_id     => ( is => 'ro', isa => Int, required => 1 );
has from_width  => ( is => 'ro', isa => Int, required => 1 );
has from_height => ( is => 'ro', isa => Int, required => 1 );
has from_digest => ( is => 'ro', isa => Str, required => 1 );

# resized image
has to_id     => ( is => 'ro', isa => Int, required => 1 );
has to_width  => ( is => 'ro', isa => Int, required => 1 );
has to_height => ( is => 'ro', isa => Int, required => 1 );
has to_digest => ( is => 'ro', isa => Str, required => 1 );

# what we send/use
has used_name   => ( is => 'ro', isa => Str, required => 1 );
has used_width  => ( is => 'ro', isa => Int, required => 1 );
has used_height => ( is => 'ro', isa => Int, required => 1 );
has used_digest => ( is => 'ro', isa => Str, required => 1 );

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => "image %i (%ix%i, digest %s) resized to image %i (%ix%i, digest %s), used %s (%ix%i, digest %s)",
    attributes => [qw/
        from_id   from_width from_height from_digest
        to_id     to_width   to_height   to_digest
        used_name used_width used_height used_digest
    /],
};

__PACKAGE__->meta->make_immutable;
1;
