use strict;
use warnings;

use Test::More;
use Test::Exception;
use Data::Dumper;

do_test(
    "state51::Logger::Event::MailoutOpened",
    {
        retailer_id => 123456,
        role_id => 654321,
        mailout_id => 321456,
    },
);

do_test(
    "state51::Logger::Event::MailoutOpened",
    {
        role_id => 123456,
        mailout_id => 654321,
    },
);

do_test(
    "state51::Logger::Event::MogileRead",
    {
        dkey => "foo",
        first_block_time => 1,
        complete_time => 2,
        size => 3,
        url => 'xxx',
        downloader => "platform",
    },
);

do_test(
    "state51::Logger::Event::MogileFail",
    {
        dkey => "foo",
        reason => 'sha1',
        downloader => "platform",
        url => 'xxx',
    },
);

do_test(
    "state51::Logger::Event::MogileWriteFail",
    {
        dkey => "foo",
        bytes_sent => 1,
        total_bytes => 2,
        client => "callback",
        time_elapsed => 3.3,
        url => 'xxx',
    },
);

do_test(
    "state51::Logger::Event::MogileWrite",
    {
        dkey => "foo",
        total_bytes => 1,
        connect_time => 2,
        time_elapsed => 3.3,
        client => "callback",
        url => 'xxx',
    }
);

do_test(
    "state51::Logger::Event::CephFail",
    {
        dkey => "foo",
        url => 'xxx',
    },
);

do_test(
    "state51::Logger::Event::CephWriteFail",
    {
        dkey => "foo",
        bytes_sent => 1,
        total_bytes => 2,
        time_elapsed => 3.3,
        url => 'xxx',
    },
);

do_test(
    "state51::Logger::Event::CephWrite",
    {
        dkey => "foo",
        total_bytes => 1,
        connect_time => 2,
        time_elapsed => 3.3,
        url => 'xxx',
    }
);

do_test(
    "state51::Logger::Event::ExportBatchUploadStarted",
    {
        bytes => 123,
        batch => 123,
        request => 123,
        tla => "xyz",
        protocol => "ftp",
        hostname => "fdsfdsfd",
        port => 21,
        raised_by_org_id => 213231,
        raised_by_org_name => "gary",
    }
);

do_test(
    "state51::Logger::Event::ExportBatchUploadFailed",
    {
        bytes => 123,
        batch => 123,
        request => 123,
        tla => "xyz",
        protocol => "ftp",
        hostname => "fdsfdsfd",
        port => 21,
        duration => 1.23,
        raised_by_org_id => 213231,
        raised_by_org_name => "gary",
    }
);

do_test(
    "state51::Logger::Event::ExportBatchUploadComplete",
    {
        bytes => 123,
        batch => 123,
        request => 123,
        tla => "xyz",
        protocol => "ftp",
        hostname => "fdsfdsfd",
        port => 21,
        duration => 1.23,
        raised_by_org_id => 213231,
        raised_by_org_name => "gary",
    }
);

do_test(
    "state51::Logger::Event::LockClaimed",
    {
        name => "foo",
    }
);

do_test(
    "state51::Logger::Event::LockFailed",
    {
        name => "foo",
    }
);

do_test(
    "state51::Logger::Event::LockReleased",
    {
        name => "foo",
        duration => 1.23,
    }
);

do_test(
    "state51::Logger::Event::Greedbag::MailCustomer",
    {
        order_no => 231231231,
        retailer_id => 213231231,
        email_to => 'andy@state51.co.uk',
        subject => "hi",
        message => "Geese are groovy."
    }
);

do_test(
    "state51::Logger::Event::Greedbag::MailOpened",
    {
        retailer_id => 213231231,
        email_to => 'andy@state51.co.uk',
        mailout_identifier => 'moosen007',
        role_id => 213321321,
    }
);

do_test(
    "state51::Logger::Event::MailService::Resubscribe",
    {
        mailservice_id => 10129920,
        mailservice_name => 'The MailService Name',
        role_id => 12599201,
    }
);

done_testing();

exit 0;

sub do_test {
    my ($class, @tests) = @_;
    note "Testing $class\n";
    use_ok $class;

    my $i = 0;
    foreach my $attr (@tests) {
        note "$class part $i";
        my $obj;
        lives_ok {
            $obj = $class->new(%{ $attr });
        };
        isa_ok($obj, $class);
        lives_ok {
            diag $obj->as_string();
        };
        lives_ok {
            diag Dumper($obj->pack());
        };
        $i++;
    }
    return;
}
