use strict;
use warnings;
use Test::More;

$ENV{PLATFORM_LOG_BACKENDS} = "TestRabbit";

my @rnd = ('a'..'z',0..9);
my $exchange_name = 'test_' . join '', map { $rnd[rand @rnd] } 0..9;
my $mq_user = $ENV{PLATFORM_MQ_USER} || "guest";
my $mq_password = $ENV{PLATFORM_MQ_PASSWORD} || "guest";
my $mq_host = $ENV{PLATFORM_MQ_SERVER} || "localhost";

{
    package state51::Logger::TestRabbit;
    use Moose;

    extends 'state51::Logger::RabbitMQ';

    override default_exchange => sub { $exchange_name };

    package state51::Logger::Event::Generic::Test;
    use Moose;

    extends 'state51::Logger::Event::Generic';

    sub routing_key { 'foobar' }
    override append_routing_key => sub { '' };
    override routing_key_filter => sub { 'foobar' };
}

use Net::AMQP::RabbitMQ;

my $mq = Net::AMQP::RabbitMQ->new();
my $channel_id = 1;
$mq->connect($mq_host, { user => $mq_user, password => $mq_password });
$mq->channel_open($channel_id);

require state51::Logger;
$mq->exchange_declare($channel_id, $exchange_name);
my ($queue_name, $messages, $consumers) = $mq->queue_declare($channel_id, 'firehose');
diag "$messages waiting messages, $consumers consumers\n";
$mq->queue_bind($channel_id, $queue_name, $exchange_name, 'foobar');
diag "consumer $channel_id, $queue_name, $exchange_name, 'foobar'";

my $consumer_tag = $mq->consume($channel_id, $queue_name);

use Data::Dumper;
diag Dumper($mq);

my $i = 0;
my $got_msg = sub {
    ok 1, 'Got message';
    $i++;
};

state51::Logger->new->pre_fork;
my $pid = fork;
if ($pid) { #Parent
    alarm(10);
    for (1..10) {
        my $msg = $mq->recv(1);
        $got_msg->($msg) if $msg;
        last if $i >=2;
        sleep 1;
    }
    is $i, 2, "both messages received";
    alarm(0);
    done_testing;
}
else { # Child
    require POSIX;
    sleep 2;
    my $mqw = Net::AMQP::RabbitMQ->new();
    $mqw->connect($mq_host, { user => $mq_user, password => $mq_password });
    $mqw->channel_open($channel_id);
    $mqw->exchange_declare($channel_id, $exchange_name);
    $mqw->publish($channel_id, 'foobar', 'test message',
        { exchange => $exchange_name });
    $mqw->disconnect;
    sleep 2;
    local $ENV{PLATFORM_MQ_SERVER}   = $mq_host;
    local $ENV{PLATFORM_MQ_USER}     = $mq_user;
    local $ENV{PLATFORM_MQ_PASSWORD} = $mq_password;
    state51::Logger->new()
        ->message(level => "info", message => [
            state51::Logger::Event::Generic::Test->new(message => ["foo"]),
        ]);
    sleep 5;
    POSIX::_exit(0);
}

